import { Request, Response } from "express";
import GenericService from "../services/GenericService";
import BaseEntity from "../entities/baseEntity";
import GenericFactory from "../factories/GenericFactory";
import { GenericViewModel } from "../viewmodels/GenericModel";

export default class GenericController<
  Service extends GenericService<Repository>,
  Factory extends GenericFactory<Repository, ViewModel>,
  Repository extends BaseEntity,
  ViewModel extends GenericViewModel
> {
  constructor(protected readonly service: Service, protected readonly factory: Factory) {}

  async getAll(req: Request, res: Response): Promise<void> {
    let data: Array<Repository> = await this.service.getAll();
    res.json(this.factory.mapArray(data));
  }

  async getById(req: Request, res: Response): Promise<void> {
    let id: string = req.params.id;
    let data: Repository = await this.service.getById(id);
    res.json(this.factory.mapSingle(data));
  }

  async create(req: Request, res: Response): Promise<void> {
    let insert = await this.service.insert(req.body);
    res.json(insert.identifiers);
  }

  async update(req: Request, res: Response): Promise<void> {
    let update = await this.service.insert(req.body);
    res.json(update.identifiers);
  }

  async delete(req: Request, res: Response): Promise<void> {
    let id: string = req.params.id;
    await this.service.delete(id);
    res.sendStatus(204);
  }
}
