import { DeleteResult, EntityTarget, InsertResult, UpdateResult } from "typeorm";
import { dataSource } from "../data-source";
import BaseEntity from "../entities/baseEntity";
import { QueryDeepPartialEntity } from "typeorm/query-builder/QueryPartialEntity";

export default class GenericService<Repository extends BaseEntity> {
  constructor(protected readonly entity: EntityTarget<Repository>) {}

  async getAll(): Promise<Repository[]> {
    return await dataSource.getRepository(this.entity).createQueryBuilder().getMany();
  }

  async getById(rowId: string): Promise<Repository> {
    return await dataSource.getRepository(this.entity).createQueryBuilder().where({ id: rowId }).getOneOrFail();
  }

  async insert(
    value: QueryDeepPartialEntity<Repository> | QueryDeepPartialEntity<Repository>[]
  ): Promise<InsertResult> {
    return await dataSource.createQueryBuilder().insert().into(this.entity).values(value).execute();
  }

  async update(value: QueryDeepPartialEntity<Repository>): Promise<UpdateResult> {
    return await dataSource.createQueryBuilder().update(this.entity).set(value).execute();
  }

  async delete(rowId: string): Promise<DeleteResult> {
    return await dataSource.createQueryBuilder().delete().from(this.entity).where({ id: rowId }).execute();
  }
}
